/*
  TGAFile.h

  TGA file definitions
*/

typedef enum tcmap_type
{
  nopalette = 0,
  palette   = 1
};

typedef enum timage_type
{
  uncompr_palette =  1,
  uncompr_rgb     =  2,
  compr_palette   =  9,
  compr_rgb       = 10
};

#pragma pack(1)
typedef struct
{
  unsigned char      ident_len;
  unsigned char      cmap_type;
  unsigned char      image_type;
  unsigned short int cmap_origin;
  unsigned short int cmap_len;
  unsigned char      centry_size;
  unsigned short int image_xorg;
  unsigned short int image_yorg;
  unsigned short int image_width;
  unsigned short int image_height;
  unsigned char      pixel_size;
  struct
  {
    unsigned char AttBits :4;  // 4 bits, number of attribute bits per pixel
    unsigned char Rsrvd   :1;  // 1 bit, reserved
    unsigned char OrgBit  :1;  // 1 bit, origin: 0=lower left, 1=upper left
    unsigned char IntrLve :2;  // 2 bits, interleaving flag
  } image_discr;
} TGAFILEHEADER;

#pragma pack()

#ifndef WINVER
typedef unsigned char BYTE;
typedef struct tagRGBQUAD
{
  BYTE rgbBlue;
  BYTE rgbGreen;
  BYTE rgbRed;
  BYTE rgbReserved;
} RGBQUAD;
#endif // #ifndef WINVER

class TGAFile
{
protected:
  TGAFILEHEADER *m_pHeader;
  unsigned long  m_dwWidthDWORDAligned;
  unsigned char *m_pData;
  int            m_nBytesPerPixel;
  bool           m_bInMemory;                                         // Shall the file be loaded into memory?
  FILE          *m_pInputFile;
  double         m_zScale;
public:
  TGAFile(bool bInMemory = true);
  ~TGAFile();
  void GetColor(const int x, const int y, RGBQUAD* pColor);             // Get the RGBQUAD value at (x, y)
  void GetColor(const unsigned int x, const unsigned int y, RGBQUAD* pColor); // Get the RGBQUAD value at (x, y)
  void GetColor(double x, double y, RGBQUAD* pColor);                   // Get the RGBQUAD value at (x, y).
                                                                        // A bilinear interpolation is done.
  void GetColorFromFile(const int x, const int y, RGBQUAD* pColor);     // Same as GetColor, but from
                                                                        // a file, not from the memory.
  long GetElevationUnscaled(const int x, const int y);                  // Get the elevation value at (x, y).
double GetElevationScaled(const int x, const int y);                    // Get the elevation value at (x, y).
  long GetElevationFromFileUnscaled(const int x, const int y);          // Get the elevation value at (x, y).
double GetElevationFromFileScaled(const int x, const int y);            // Get the elevation value at (x, y).
  long GetElevationUnscaled(double x, double y);                        // Get the elevation value at (x, y).
double GetElevationScaled(double x, double y);                          // Get the elevation value at (x, y).
                                                                        // A bilinear interpolation is done.
   int Open(const char *szFilename);                                    // Open the TGA file using szFilename
   int New(const int nWidth, const int nHeight);                        // Create a new (all white) file
  void Close();                                                         // Close the TGA file.
  void SetColor(const int x, const int y, RGBQUAD* pColor);             // Set the color at (x, y)
  void SetElevationScaled(const int x, const int y, const double elevation); // Set the elevation value at (x, y)
  void SetElevationUnscaled(const int x, const int y, const long elevation); // Set the elevation value at (x, y)
   int Write(const char* szFilename);                                   // Write the TGA file (if it is open)
   int GetImageWidth() { return m_pHeader->image_width; }               // Get width in pixels
   int GetImageHeight() { return m_pHeader->image_height; }             // Get height in pixels
double GetZScale();
  void SetZScale(double zScale);
};
