// geoDataClientDialog.cpp: implementation file
//

#include "stdafx.h"

#include <afxtempl.h>
#include <math.h>
#include <float.h>

#include <list>
#include "Waypoint.h"

#include "geoDataClient.h"
#include "geoDataClientDialog.h"
#include "AboutDialog.h"
#include "HTTPRequest.h"

#include "GeoCoderQueryThread.h"
#include "SeamlessQueryThread.h"

#include "SettingsDialog.h"
#include "geoDataClientDialog.h"
#include "URL.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

// CgeoDataClientDialog dialog

CgeoDataClientDialog::CgeoDataClientDialog(CWnd* pParent /*=NULL*/)
	: CDialog(CgeoDataClientDialog::IDD, pParent)
  , strAddress(_T(""))
  , quadLeft(-121.1)
  , quadRight(-121.0)
  , quadTop(42.1)
  , quadBottom(42.0)
{
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
  bCancel = FALSE;
  allowedToEnableProceedButton = true; // If appropriate, that button may be enabled.
  maxListboxTextExtent = 0;
  quadSize = 1.0;
  interpolate = false;
  alignQuadToGrid = false;
  removeIntermediateFiles = true;
  lineThickness = 1;
  zScale = 0.02;
  nHTTPTimeout = 120;
  bUseWaypoints = FALSE;
  outputFileNamesFromCoordinates = FALSE;
  allowedToUpdate = true;
  strWaypointFile.Empty();
  waypointList.clear();
	char *gdcScanner = getenv("gdcScanner");
	if(gdcScanner)
	{
		if(!stricmp(gdcScanner, "yes"))
		{
			quadSize = 0.25;
			alignQuadToGrid = TRUE;
			outputFileNamesFromCoordinates = TRUE;
		}
	}
}

void CgeoDataClientDialog::DoDataExchange(CDataExchange* pDX)
{
  CDialog::DoDataExchange(pDX);
  DDX_Control(pDX, IDC_LIST, listBox);
  DDX_Control(pDX, IDCLOSE, closeButton);
  DDX_Control(pDX, IDC_CANCELOPERATION, cancelButton);
  DDX_Control(pDX, IDC_PROCEED, proceedButton);
  DDX_Text(pDX, IDC_EDIT_ADDRESS, strAddress);
  DDX_Control(pDX, IDC_PROGRESS, progressControl);
  DDX_Control(pDX, IDC_STATEINDICATOR, stateIndicator);
  DDX_Text(pDX, IDC_EDIT_LEFT, quadLeft);
  DDX_Text(pDX, IDC_EDIT_RIGHT, quadRight);
  DDX_Text(pDX, IDC_EDIT_TOP, quadTop);
  DDX_Text(pDX, IDC_EDIT_BOTTOM, quadBottom);
}

BEGIN_MESSAGE_MAP(CgeoDataClientDialog, CDialog)
	ON_WM_SYSCOMMAND()
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	//}}AFX_MSG_MAP
  ON_BN_CLICKED(IDC_GEOCODE, OnBnClickedGeocode)
  ON_BN_CLICKED(IDCLOSE, OnBnClickedClose)
  ON_BN_CLICKED(IDC_CANCELOPERATION, OnBnClickedCancel)
  ON_LBN_SELCHANGE(IDC_LIST, OnLbnSelchangeList)
  ON_BN_CLICKED(IDC_PROCEED, OnBnClickedProceed)
  ON_BN_CLICKED(IDC_COPY, OnBnClickedCopy)
  ON_BN_CLICKED(IDC_SETTINGS, OnBnClickedSettings)
  ON_EN_CHANGE(IDC_EDIT_LEFT, OnEnChangeEditLeft)
  ON_EN_CHANGE(IDC_EDIT_TOP, OnEnChangeEditTop)
  ON_BN_CLICKED(IDC_BUTTON_GEOCODER_US, OnBnClickedButtonGeocoderUs)
  ON_BN_CLICKED(IDC_BUTTON_USGS_GOV, OnBnClickedButtonUsgsGov)
  ON_BN_CLICKED(IDC_BUTTON_CENSUS_GOV, OnBnClickedButtonCensusGov)
END_MESSAGE_MAP()


// CgeoDataClientDialog message handlers

BOOL CgeoDataClientDialog::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Add "About..." menu item to system menu.

	// IDM_ABOUTBOX must be in the system command range.
	ASSERT((IDM_ABOUTBOX & 0xFFF0) == IDM_ABOUTBOX);
	ASSERT(IDM_ABOUTBOX < 0xF000);

	CMenu* pSysMenu = GetSystemMenu(FALSE);
	if (pSysMenu != NULL)
	{
		CString strAboutMenu;
		strAboutMenu.LoadString(IDS_ABOUT_DIALOG);
		if (!strAboutMenu.IsEmpty())
		{
			pSysMenu->AppendMenu(MF_SEPARATOR);
			pSysMenu->AppendMenu(MF_STRING, IDM_ABOUTBOX, strAboutMenu);
		}
	}

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

	// TODO: Add extra initialization here
	
  cancelButton.EnableWindow(FALSE);
  proceedButton.EnableWindow(FALSE);

  listBox.ShowScrollBar(SB_HORZ);

  return TRUE;  // return TRUE  unless you set the focus to a control
}

void CgeoDataClientDialog::OnSysCommand(UINT nID, LPARAM lParam)
{
	if ((nID & 0xFFF0) == IDM_ABOUTBOX)
	{
		CAboutDialog dlgAbout;
		dlgAbout.DoModal();
	}
	else
	{
		CDialog::OnSysCommand(nID, lParam);
	}
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CgeoDataClientDialog::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this function to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CgeoDataClientDialog::OnQueryDragIcon()
{
	return static_cast<HCURSOR>(m_hIcon);
}

void CgeoDataClientDialog::OnBnClickedGeocode()
{
  // TODO: Add your control notification handler code here
  // Start the other thread.
  UpdateData();
  SetStateOK();
  AfxBeginThread(GeoCoderQueryThread, this);
}

void CgeoDataClientDialog::OnBnClickedClose()
{
  // TODO: Add your control notification handler code here
  OnCancel();
}

void CgeoDataClientDialog::OnBnClickedCancel()
{
  // TODO: Add your control notification handler code here
  bCancel = TRUE;
}

void CgeoDataClientDialog::FilterGeocoderUS(CString strResultText)
{
  strResultText.Replace("\n", " ");
	strResultText.Replace("\n", " ");
	strResultText.Replace("\n", " ");
	strResultText.Replace("\n", " ");
	strResultText.Replace("\n", " ");
	strResultText.Replace("\t", " ");
	strResultText.Replace("                 ", " ");
	strResultText.Replace("         ", " ");
	strResultText.Replace("     ", " ");
	strResultText.Replace("   ", " ");
	strResultText.Replace("  ", " ");
	strResultText.Replace(" =", "=");
	strResultText.Replace("= ", "=");
//  CString message;
//  message.Format(_T("DEBUG: strResultText: \"%s\""), strResultText);
//  addMessage(message);
	double  latitude = 0.0;
	double longitude = 0.0;
  CString tempString = strResultText;
  tempString = tempString.MakeLower();
//  CString message;
//  message.Format(_T("DEBUG: tempString: \"%s\""), strResultText);
//  addMessage(message);
	int latitudePointer = tempString.Find("mapcenterlat=");
	if(latitudePointer != -1)
	{
		int comma = tempString.Find(",", latitudePointer + 13);
    if(comma != -1)
    {
//      CString message;
//      message.Format(_T("DEBUG: latitudePointer = %d, comma = %d"), latitudePointer, comma);
//      addMessage(message);

//      CString message;
//      message.Format(_T("DEBUG: mapCenterLat: \"%s\""), tempString.Mid(latitudePointer + 13, comma - (latitudePointer + 13)));
//      addMessage(message);
      latitude = atof(tempString.Mid(latitudePointer + 13, comma - (latitudePointer + 13)));
    }
	}
	int longitudePointer = tempString.Find("mapcenterlon=");
	if(longitudePointer != -1)
	{
		int comma = tempString.Find(",", longitudePointer + 13);
    if(comma != -1)
    {
//      CString message;
//      message.Format(_T("DEBUG: longitudePointer = %d, comma = %d"), longitudePointer, comma);
//      addMessage(message);

//      CString message;
//      message.Format(_T("DEBUG: mapCenterLon: \"%s\""), tempString.Mid(longitudePointer + 13, comma - (longitudePointer + 13)));
//      addMessage(message);
      longitude = atof(tempString.Mid(longitudePointer + 13, comma - (longitudePointer + 13)));
    }
	}
	if((longitude == 0.0) && (latitude == 0.0))
	{
    CList<CString,CString&> messageList;
    int index1 = 0;
    int index2 = 0;
    while(index1 != -1)
    {
      index1 = tempString.Find("<td align=\"center\">", index2);
      if(index1 != -1)
      {
        index2 = tempString.Find("</td>", index1 + 19);
        if(index2 != -1)
        {
          CString entry = tempString.Mid(index1 + 19, index2 - index1 - 19);
          int index3 = entry.Find("<br />");
          if(index3 != -1)
          {
            entry = strResultText.Mid(index1 + 19, index3 - 1) + ", " + strResultText.Mid(index1 + 19 + index3 + 6, index2 - (index1 + 19 + index3 + 6));
            messageList.AddTail(entry);
          }
        }
        else
        {
          break;
        }
      }
    }
    if(messageList.GetCount() > 0)
    {
      addMessage(_T("        Multiple addresses found:"));
      POSITION pos = messageList.GetHeadPosition();
      while(pos != NULL)
      {
        CString entry("        ");
        entry += messageList.GetNext(pos) + "";
        addMessage(entry);
      }
      BlinkError();
    }
    else
    {
      addMessage(_T("        No address found."));
      BlinkError();
    }
  }
	else
	{
//    double  right;
//    double   left;
//    double    top;
//    double bottom;
    if(alignQuadToGrid)
    {
       quadRight = ceil(longitude / quadSize) * quadSize;
        quadLeft = quadRight - quadSize;
         quadTop = ceil(latitude / quadSize) * quadSize;
      quadBottom = quadTop - quadSize;
    }
    else
    {
       quadRight = longitude + quadSize * 0.5;
        quadLeft = longitude - quadSize * 0.5;
         quadTop = latitude + quadSize * 0.5;
      quadBottom = latitude - quadSize * 0.5;
    }
    CString message;
    message.Format(_T("  quad: %lf, %lf, %lf, %lf"), quadLeft, quadRight, quadTop, quadBottom);
    addMessage(message);

    // Now, set the edit controls.
    // Avoid using UpdateData(FALSE) because we are in a
    // different thread than the main Dialog's event loop (?).
    allowedToUpdate = false;
    CWnd *pLeft = GetDlgItem(IDC_EDIT_LEFT);
    message.Format(_T("%f"), quadLeft);
    if(pLeft) pLeft->SetWindowText(message);
    CWnd *pRight = GetDlgItem(IDC_EDIT_RIGHT);
    message.Format(_T("%f"), quadRight);
    if(pRight) pRight->SetWindowText(message);
    CWnd *pTop = GetDlgItem(IDC_EDIT_TOP);
    message.Format(_T("%f"), quadTop);
    if(pTop) pTop->SetWindowText(message);
    CWnd *pBottom = GetDlgItem(IDC_EDIT_BOTTOM);
    message.Format(_T("%f"), quadBottom);
    if(pBottom) pBottom->SetWindowText(message);
    allowedToUpdate = true;

    proceedButton.EnableWindow();
	if(pLeft) pLeft->EnableWindow();
	if(pTop) pTop->EnableWindow();
	}
}

void CgeoDataClientDialog::addMessage(const CString& text)
{
	CDC* pDC = GetDC();
	HFONT hFont = (HFONT)SendMessage(WM_GETFONT);
	CFont *pFont = CFont::FromHandle(hFont);
	ASSERT(pFont);
	CFont* pPrevFont = pDC->SelectObject(pFont);

	CSize newExtent = pDC->GetTextExtent(text, text.GetLength());
	if(newExtent.cx > maxListboxTextExtent)
	{
		maxListboxTextExtent = newExtent.cx;
	}

	listBox.SetHorizontalExtent(maxListboxTextExtent + 6);
	pDC->SelectObject(pPrevFont);
	ReleaseDC(pDC);

	listBox.AddString(text);
	int position = listBox.GetCount() - 1;
	listBox.SetCurSel(position);
}

void CgeoDataClientDialog::OnLbnSelchangeList()
{
	// TODO: Add your control notification handler code here
	if(listBox.GetCurSel() == -1)
	{
		// Nothing in the list yet.
		return;
	}
	CString message;
	listBox.GetText(listBox.GetCurSel(), message);
	if(message.Find("  quad: ") == 0)
	{
		if(allowedToEnableProceedButton)
		{
			proceedButton.EnableWindow();
			CWnd *pLeft = GetDlgItem(IDC_EDIT_LEFT);
			if(pLeft) pLeft->EnableWindow();
			CWnd *pTop = GetDlgItem(IDC_EDIT_TOP);
			if(pTop) pTop->EnableWindow();
		}
	}
/*
	else
	{
		proceedButton.EnableWindow(FALSE);
		CWnd *pLeft = GetDlgItem(IDC_EDIT_LEFT);
		if(pLeft) pLeft->EnableWindow(FALSE);
		CWnd *pTop = GetDlgItem(IDC_EDIT_TOP);
		if(pTop) pTop->EnableWindow(FALSE);
	}
*/
	if(message.Find("        ") == 0)
	{
		strAddress = message.Mid(9);
		int nRightPosition = strAddress.Find("");
		if(nRightPosition != -1)
		{
			strAddress = strAddress.Left(nRightPosition);
			UpdateData(FALSE);
		}
	}
}

void CgeoDataClientDialog::OnBnClickedProceed()
{
  // TODO: Add your control notification handler code here
  bCancel = false;
  SetStateOK();
  AfxBeginThread(SeamlessQueryThread, this);
}

void CgeoDataClientDialog::OnBnClickedCopy()
{
  CString text;
  CTime now = CTime::GetCurrentTime();
  text = now.FormatGmt("geoDataClientDialog log file created %A, %B %d, %Y %H:%M:%S UTC\r\n");

  int nIndex;
  for(nIndex = 0; nIndex < listBox.GetCount(); nIndex++)
  {
    CString line;
    listBox.GetText(nIndex, line);
    text = text + line + "\r\n";
  }

  // TODO: Add your control notification handler code here
  if(!OpenClipboard())
  {
    addMessage(_T("Cannot open the Clipboard"));
    HICON hIcon = AfxGetApp()->LoadIcon(IDI_STATE_ERROR);
    stateIndicator.SetIcon(hIcon);
    return;
  }
  // Remove the current Clipboard contents
  if(!EmptyClipboard())
  {
    addMessage(_T("Cannot empty the Clipboard"));
    SetStateError();
    return;
  }
  // Get the currently selected data
  HGLOBAL hglbCopy = GlobalAlloc(GMEM_MOVEABLE, (text.GetLength() + 1) * sizeof(TCHAR));
  if(hglbCopy == NULL)
  {
    addMessage(_T("Cannot allocate memory for copying to the Clipboard"));
    SetStateError();
    CloseClipboard();
  }
  // Lock the handle and copy the text to the buffer.

  LPTSTR lptstrCopy = (LPTSTR)GlobalLock(hglbCopy);
  strcpy(lptstrCopy, text);
  lptstrCopy[text.GetLength()] = (TCHAR)0;    // null character
  GlobalUnlock(hglbCopy);

  // Place the handle on the clipboard.

  if(SetClipboardData(CF_TEXT, hglbCopy) == NULL)
  {
    addMessage(_T("Unable to set Clipboard data"));
    SetStateError();
  }
  CloseClipboard();
}

void CgeoDataClientDialog::OnBnClickedSettings()
{
  // TODO: Add your control notification handler code here
  CSettingsDialog settingsDialog;
  settingsDialog.setQuadsize(quadSize);
  settingsDialog.setInterpolate(interpolate);
  settingsDialog.setAlign(alignQuadToGrid);
  settingsDialog.setRemoveIntermediateFiles(removeIntermediateFiles);
  settingsDialog.setLineThickness(lineThickness);
  settingsDialog.setZScale(zScale);
  settingsDialog.setHTTPTimeout(nHTTPTimeout);
  settingsDialog.setUseWaypoints(bUseWaypoints);
  settingsDialog.setWaypointFile(strWaypointFile);
  settingsDialog.setOutputFileNamesFromCoordinates(outputFileNamesFromCoordinates);
  if(settingsDialog.DoModal() == IDOK)
  {
    quadSize = settingsDialog.getQuadsize();
    interpolate = settingsDialog.getInterpolate();
    alignQuadToGrid = settingsDialog.getAlign();
    removeIntermediateFiles = settingsDialog.getRemoveIntermediateFiles();
    lineThickness = settingsDialog.getLineThickness();
    zScale = settingsDialog.getZScale();
    nHTTPTimeout = settingsDialog.getHTTPTimeout();
    bUseWaypoints = settingsDialog.getUseWaypoints();
	outputFileNamesFromCoordinates = settingsDialog.getOutputFileNamesFromCoordinates();
    if(bUseWaypoints)
    {
      strWaypointFile = settingsDialog.getWaypointFile();
      bool bResult = readWaypoints(strWaypointFile);
      if(bResult)
      {
        strWaypointFile.Empty();
      }
    }
    OnEnChangeEditLeft();
    OnEnChangeEditTop();
  }
}

void CgeoDataClientDialog::BlinkError()
{
  HICON hIcon = AfxGetApp()->LoadIcon(IDI_STATE_ERROR);
  stateIndicator.SetIcon(hIcon);
  Sleep(500);
  hIcon = AfxGetApp()->LoadIcon(IDI_STATE_OK);
  stateIndicator.SetIcon(hIcon);
}


void CgeoDataClientDialog::SetStateError()
{
  HICON hIcon = AfxGetApp()->LoadIcon(IDI_STATE_ERROR);
  stateIndicator.SetIcon(hIcon);
  proceedButton.EnableWindow(TRUE);
}


void CgeoDataClientDialog::SetStateWarn()
{
  HICON hIcon = AfxGetApp()->LoadIcon(IDI_STATE_WARN);
  stateIndicator.SetIcon(hIcon);
}


void CgeoDataClientDialog::SetStateOK()
{
  HICON hIcon = AfxGetApp()->LoadIcon(IDI_STATE_OK);
  stateIndicator.SetIcon(hIcon);
}

void CgeoDataClientDialog::OnEnChangeEditLeft()
{
  // TODO:  If this is a RICHEDIT control, the control will not
  // send this notification unless you override the CDialog::OnInitDialog()
  // function and call CRichEditCtrl().SetEventMask()
  // with the ENM_CHANGE flag ORed into the mask.

  // TODO:  Add your control notification handler code here
  if(allowedToUpdate)
  {
    CWnd *pLeft = GetDlgItem(IDC_EDIT_LEFT);
    CString strLeft;
    if(pLeft) pLeft->GetWindowText(strLeft);
    quadLeft = atof(strLeft);

    quadRight = quadLeft + quadSize;
    CWnd *pRight = GetDlgItem(IDC_EDIT_RIGHT);
    CString strRight;
    strRight.Format(_T("%f"), quadRight);
    if(pRight) pRight->SetWindowText(strRight);
    if(allowedToEnableProceedButton)
    {
      proceedButton.EnableWindow();
	CWnd *pLeft = GetDlgItem(IDC_EDIT_LEFT);
	if(pLeft) pLeft->EnableWindow();
	CWnd *pTop = GetDlgItem(IDC_EDIT_TOP);
	if(pTop) pTop->EnableWindow();
    }
  }
}

void CgeoDataClientDialog::OnEnChangeEditTop()
{
  // TODO:  If this is a RICHEDIT control, the control will not
  // send this notification unless you override the CDialog::OnInitDialog()
  // function and call CRichEditCtrl().SetEventMask()
  // with the ENM_CHANGE flag ORed into the mask.

  // TODO:  Add your control notification handler code here
  if(allowedToUpdate)
  {
    CWnd *pTop = GetDlgItem(IDC_EDIT_TOP);
    CString strTop;
    if(pTop) pTop->GetWindowText(strTop);
    quadTop = atof(strTop);

    quadBottom = quadTop - quadSize;
    CWnd *pBottom = GetDlgItem(IDC_EDIT_BOTTOM);
    CString strBottom;
    strBottom.Format(_T("%f"), quadBottom);
    if(pBottom) pBottom->SetWindowText(strBottom);
    if(allowedToEnableProceedButton)
    {
      proceedButton.EnableWindow();
	CWnd *pLeft = GetDlgItem(IDC_EDIT_LEFT);
	if(pLeft) pLeft->EnableWindow();
	CWnd *pTop = GetDlgItem(IDC_EDIT_TOP);
	if(pTop) pTop->EnableWindow();
    }
  }
}

void CgeoDataClientDialog::OnBnClickedButtonGeocoderUs()
{
  // TODO: Add your control notification handler code here
  CURL url;
  url.Open(_T("http://www.geocoder.us/"), false);
}

void CgeoDataClientDialog::OnBnClickedButtonUsgsGov()
{
  // TODO: Add your control notification handler code here
  CURL url;
  url.Open(_T("http://www.usgs.gov/"), false);
}

void CgeoDataClientDialog::OnBnClickedButtonCensusGov()
{
  // TODO: Add your control notification handler code here
  CURL url;
  url.Open(_T("http://www.census.gov/"), false);
}
